// Copyright Joan Sol Roo and Christian Richardt (7 August 2014)
//
// solroo@gmail.com
// christian@richardt.name
//
// This software is a computer program whose purpose is to compute temporally
// coherent stereoscopic 3D videos from anaglyph 3D input videos.
//
// This software is governed by the CeCILL license under French law and
// abiding by the rules of distribution of free software. You can use, modify
// and/or redistribute the software under the terms of the CeCILL license as
// circulated by CEA, CNRS and INRIA at "http://www.cecill.info".
//
// As a counterpart to the access to the source code and rights to copy,
// modify and redistribute granted by the license, users are provided only
// with a limited warranty and the software's author, the holder of the
// economic rights, and the successive licensors have only limited liability.
//
// In this respect, the user's attention is drawn to the risks associated
// with loading, using, modifying and/or developing or reproducing the
// software by the user in light of its specific status of free software,
// that may mean that it is complicated to manipulate, and that also
// therefore means that it is reserved for developers and experienced
// professionals having in-depth computer knowledge. Users are therefore
// encouraged to load and test the software's suitability as regards their
// requirements in conditions enabling the security of their systems and/or
// data to be ensured and, more generally, to use and operate it in the same
// conditions as regards security.
//
// The fact that you are presently reading this means that you have had
// knowledge of the CeCILL license and that you accept its terms.

#include <ctime>

#include "DebugTools.h"

using namespace cv;
using namespace std;


template <class TMat>
void debug(string name, TMat mat, int range)
{
	for (int y = 0; y < range; y++)
	{
		for (int x = 0; x < range; x++)
		{
			cout << name << "(" << (y + 1) << ", " << (x + 1) << ") =" << mat(y, x) << endl;
		}
	}
}

double tic()
{
	clock_t begin = clock();
	return begin;
}

double toc(double tic)
{
	clock_t end = clock();
	double elapsed_secs = double(end - tic) / CLOCKS_PER_SEC;
	return elapsed_secs;
}


Mat showDiff(InputArray image, InputArray error, const string& name, int diffchannel)
{
	Mat error_gray, result;
	cvtColor(error, error_gray, CV_BGR2GRAY);
	vector<Mat> channels(3);
	split(image, channels);
	channels[diffchannel] = channels[diffchannel] + error_gray;
	merge(channels,result);
	imshow(name, result);
	return result;
}

Mat showError(const Mat& result, const Mat& expected_result, const string& name, int scale)
{
	Mat error = abs(result - expected_result) * scale;
	imshow(name, error);
	return error;
}
