// Copyright Joan Sol Roo and Christian Richardt (7 August 2014)
//
// solroo@gmail.com
// christian@richardt.name
//
// This software is a computer program whose purpose is to compute temporally
// coherent stereoscopic 3D videos from anaglyph 3D input videos.
//
// This software is governed by the CeCILL license under French law and
// abiding by the rules of distribution of free software. You can use, modify
// and/or redistribute the software under the terms of the CeCILL license as
// circulated by CEA, CNRS and INRIA at "http://www.cecill.info".
//
// As a counterpart to the access to the source code and rights to copy,
// modify and redistribute granted by the license, users are provided only
// with a limited warranty and the software's author, the holder of the
// economic rights, and the successive licensors have only limited liability.
//
// In this respect, the user's attention is drawn to the risks associated
// with loading, using, modifying and/or developing or reproducing the
// software by the user in light of its specific status of free software,
// that may mean that it is complicated to manipulate, and that also
// therefore means that it is reserved for developers and experienced
// professionals having in-depth computer knowledge. Users are therefore
// encouraged to load and test the software's suitability as regards their
// requirements in conditions enabling the security of their systems and/or
// data to be ensured and, more generally, to use and operate it in the same
// conditions as regards security.
//
// The fact that you are presently reading this means that you have had
// knowledge of the CeCILL license and that you accept its terms.

#pragma once

#include <opencv2/opencv.hpp>


// Computes the "Mean squared error" between two matrices 
double getMSE( const cv::Mat3b& I1, const cv::Mat3b& I2);

// Computes the "Root mean squared deviation" between two matrices
double getRMSD(const cv::Mat3b& I1, const cv::Mat3b& I2);

// Computes the "Peak signal-to-noise ratio" between two matrices
// Based on the example provided by OpenCV, which can be found at:
//     samples/cpp/tutorial_code/HighGUI/video-input-psnr-ssim/
// More information regarding this implementation can be found here:
//     http://docs.opencv.org/doc/tutorials/highgui/video-input-psnr-ssim/video-input-psnr-ssim.html
double getPSNR(const cv::Mat& I1, const cv::Mat& I2);

//  Computes the "Structural similarity" between two matrices
// Based on the example provided by OpenCV, which can be found at:
//     samples/cpp/tutorial_code/HighGUI/video-input-psnr-ssim/
// More information regarding this implementation can be found here:
//     http://docs.opencv.org/doc/tutorials/highgui/video-input-psnr-ssim/video-input-psnr-ssim.html
cv::Scalar getMSSIM( const cv::Mat& I1, const cv::Mat& I2);
