// Copyright Joan Sol Roo and Christian Richardt (7 August 2014)
//
// solroo@gmail.com
// christian@richardt.name
//
// This software is a computer program whose purpose is to compute temporally
// coherent stereoscopic 3D videos from anaglyph 3D input videos.
//
// This software is governed by the CeCILL license under French law and
// abiding by the rules of distribution of free software. You can use, modify
// and/or redistribute the software under the terms of the CeCILL license as
// circulated by CEA, CNRS and INRIA at "http://www.cecill.info".
//
// As a counterpart to the access to the source code and rights to copy,
// modify and redistribute granted by the license, users are provided only
// with a limited warranty and the software's author, the holder of the
// economic rights, and the successive licensors have only limited liability.
//
// In this respect, the user's attention is drawn to the risks associated
// with loading, using, modifying and/or developing or reproducing the
// software by the user in light of its specific status of free software,
// that may mean that it is complicated to manipulate, and that also
// therefore means that it is reserved for developers and experienced
// professionals having in-depth computer knowledge. Users are therefore
// encouraged to load and test the software's suitability as regards their
// requirements in conditions enabling the security of their systems and/or
// data to be ensured and, more generally, to use and operate it in the same
// conditions as regards security.
//
// The fact that you are presently reading this means that you have had
// knowledge of the CeCILL license and that you accept its terms.

#include <stdlib.h>
#include <math.h>

#include "FlowUtils.h"
#include "RenderFlow.h"
#include "colorcode.h"

using namespace cv;


Scalar computeColor(float fx, float fy)
{
	uchar pix[3];
	computeColor(fx, fy, pix);
	return Scalar(pix[0], pix[1], pix[2]);
}

Mat3b renderFlow(const Mat2f& optical_flow, Mat3b optical_flow_render, float scale, bool show_directions)
{
	if(optical_flow_render.empty())
		optical_flow_render = cv::Mat3b(optical_flow.rows, optical_flow.cols);

	#pragma omp parallel for
	for(int y = 0; y < optical_flow_render.rows; y++)
	{
		for(int x = 0; x < optical_flow_render.cols; x++)
		{
			const Vec2f& fxy = optical_flow.at<Vec2f>(y, x);
			if(fxy[0] != -kFLOW_UNKNOWN[0] && fxy[1] != -kFLOW_UNKNOWN[1])
			{
				Scalar vMiddlebury = computeColor(fxy[0] * scale, fxy[1] * scale);
				line(optical_flow_render, Point(x, y), Point(x, y), vMiddlebury);
			}
			else
			{
				line(optical_flow_render, Point(x, y), Point(x, y), Scalar(0, 0, 0));
			}
		}
	}

	if(show_directions)
	{
		int step = 20;
		Scalar color(0, 0, 0);
		for(int y = 0; y < optical_flow_render.rows; y += step)
		{
			for(int x = 0; x < optical_flow_render.cols; x += step)
			{
				const Point2f& fxy = optical_flow.at<Point2f>(y, x);
				circle(optical_flow_render, Point(x,y), 2, color, -1);
				line(optical_flow_render, Point(x,y), Point(cvRound(x + fxy.x), cvRound(y + fxy.y)), color);
			}
		}
	}

	return optical_flow_render;
}