// Copyright Joan Sol Roo and Christian Richardt (7 August 2014)
//
// solroo@gmail.com
// christian@richardt.name
//
// This software is a computer program whose purpose is to compute temporally
// coherent stereoscopic 3D videos from anaglyph 3D input videos.
//
// This software is governed by the CeCILL license under French law and
// abiding by the rules of distribution of free software. You can use, modify
// and/or redistribute the software under the terms of the CeCILL license as
// circulated by CEA, CNRS and INRIA at "http://www.cecill.info".
//
// As a counterpart to the access to the source code and rights to copy,
// modify and redistribute granted by the license, users are provided only
// with a limited warranty and the software's author, the holder of the
// economic rights, and the successive licensors have only limited liability.
//
// In this respect, the user's attention is drawn to the risks associated
// with loading, using, modifying and/or developing or reproducing the
// software by the user in light of its specific status of free software,
// that may mean that it is complicated to manipulate, and that also
// therefore means that it is reserved for developers and experienced
// professionals having in-depth computer knowledge. Users are therefore
// encouraged to load and test the software's suitability as regards their
// requirements in conditions enabling the security of their systems and/or
// data to be ensured and, more generally, to use and operate it in the same
// conditions as regards security.
//
// The fact that you are presently reading this means that you have had
// knowledge of the CeCILL license and that you accept its terms.

#pragma once

#include <opencv2/opencv.hpp>
#include <FeatureFlow.h>

using cv::Vec3f;
using std::string;
using std::vector;


struct Globals
{
	Globals()
	: extension(".png")
	, max_disparity(255.0f / 4)
	, has_gt_disparities(false)
	, load_disparities(false)
	, filter_disparities(true)
	//, left_factor(0, 0.7, 0.3); // half-color anaglyph
	//, left_factor(0.114, 0.587, 0.299); // optimized anaglyph
	, left_factor(0, 0, 1) // full-color anaglyph
	, time_disparity(0.)
	, time_flow(0.)
	, time_color(0.)
	, time_filtering(0.)
	{}

	FrameSequence leftFrames;
	FrameSequence rightFrames;

	//vector<KeyFrame> leftFlow;

	string video_name;
	string input_path;
	string output_path;
	string extension;

	int num_digits;      // Number of digits in the frame number (e.g. 4 for "0017")
	int start_at;        // First frame
	int frames;          // Frame count
	int step;            // Frame step (1 = every frame)

	cv::Size max_size;   // Maximum image resolution. If the frames are bigger, they will be resized to the maximum valid size, keeping the aspect ratio.
	float max_disparity; // Normalization value: maximum disparity
	float flow_scale;    // Normalization value: scale factor for optical flow
	
	bool has_gt_disparities; // Indicates if the video has ground-truth disparity maps
	bool load_disparities;
	bool filter_disparities;

	Vec3f left_factor;

	double time_disparity;
	double time_flow;
	double time_color;
	double time_filtering;
};
