#ifndef _ImageIO_h
#define _ImageIO_h

#include "cv.h"
#include "highgui.h"
#include "Image.h" 

class ImageIO
{
public:
	enum ImageType{standard, derivative, normalized};
	ImageIO(void);
	~ImageIO(void);
public:
	template <class T>
	static bool loadImage(const char* filename,T*& pImagePlane,int& width,int& height, int& nchannels);
	template <class T>
	static bool saveImage(const char* filename,const T* pImagePlane,int width,int height, int nchannels,ImageType imtype = standard);
	template <class T>
	static cv::Mat image2mat(const T* pImagePlane,int width,int height, int nchannels,ImageType imtype = standard);
	template <class TI, class TO>
	static bool mat2image(const cv::Mat& im, TO *&pImagePlane, int &width, int &height, int &nchannels);

};

template <class TI, class TO>
bool ImageIO::mat2image(const cv::Mat& im, TO *&pImagePlane, int &width, int &height, int &nchannels)
{
	if(im.data == NULL) // if allocation fails
		return false;
	//if(im.type()!=CV_8UC1 && im.type()!=CV_8UC3 && im.type()!=CV_8UC4) // we only support three types of image information for now
	//	return false;
	width = im.size().width;
	height = im.size().height;
	nchannels = im.channels();
	pImagePlane = new TO[width*height*nchannels];
	
	if(typeid(TI) == typeid(TO) && typeid(TO) == typeid(unsigned char))
	{
		for(int i = 0;i<height;i++)
			memcpy(pImagePlane+i*im.step,im.data+i*im.step,width*nchannels);
		return false;
	}

	// check whether the type is float point
	bool IsFloat=false;
	if(typeid(TI)==typeid(double) || typeid(TI)==typeid(float) || typeid(TI)==typeid(long double))
		IsFloat=true;
	
	#pragma omp parallel for
	for(int y = 0; y < height; ++y)
	{
		for(int x = 0; x < width; ++x)
		{
			int offset1 = (y*width+x)*nchannels;
			int offset2 = (y*width+x)*nchannels;
			for(int c=0;c<nchannels;c++)
			{
				//cout << ((TI*)(im.data))[offset2+c] << endl;
				if(IsFloat)
					pImagePlane[offset1+c] = ((TI*)(im.data))[offset2+c] * 255;
				else
					pImagePlane[offset1+c] = im.data[offset2+c];
			}
		}
	}
	return true;
}
template <class T>
bool ImageIO::loadImage(const char *filename, T *&pImagePlane, int &width, int &height, int &nchannels)
{
	cv::Mat im = cv::imread(filename);
	return mat2image<T,T>(im, pImagePlane, width, height, nchannels);
}

template <class T>
cv::Mat ImageIO::image2mat(const T* pImagePlane,int width,int height, int nchannels,ImageType imtype)
{
	cv::Mat im;
	//switch(nchannels){
	//	case 1:
	//		im.create(height,width,CV_8UC1);
	//		break;
	//	case 3:
	//		im.create(height,width,CV_8UC3);
	//		break;
	//	default:
	//		return cv::Mat();
	//}

	// check whether the type is float point
	bool IsFloat=false;
	if(typeid(T)==typeid(double) || typeid(T)==typeid(float) || typeid(T)==typeid(long double))
	{	
		im = cv::Mat(height,width, CV_MAKETYPE(CV_64F, nchannels), (void*)pImagePlane);
		IsFloat=true;
	}else
	{
		//im.create(height,width, CV_MAKETYPE(CV_8U, nchannels));
		im = cv::Mat(height,width, CV_MAKETYPE(CV_8U, nchannels), (void*)pImagePlane);
	}
	/*T Max,Min;
	int nElements = width*height*nchannels;
	switch(imtype){
		case standard:
			break;
		case derivative:
			// find the max of the absolute value
			Max = pImagePlane[0];
			if(!IsFloat)
				for(int i = 0;i<nElements;i++)
					Max = __max(Max,abs(pImagePlane[i]));
			else
				for(int i=0;i<nElements;i++)
					Max = __max(Max,fabs((double)pImagePlane[i]));
			Max*=2;
			break;
		case normalized:
			Max = Min = pImagePlane[0];
			for(int i = 0;i<nElements;i++)
			{
				Max = __max(Max,pImagePlane[i]);
				Min = __min(Min,pImagePlane[i]);
			}
			break;
	}
	if(typeid(T) == typeid(unsigned char) && imtype == standard)
	{
		for(int i = 0;i<height;i++)
			memcpy(im.data+i*im.step,pImagePlane+i*im.step,width*nchannels);
	}
	else
	{
		for(int i =0;i<height;i++)
		{
			int offset1 = i*width*nchannels;
			int offset2 = i*im.step;
			for(int j=0;j<im.step;j++)
			{
				switch(imtype){
					case standard:
						if(IsFloat)
							im.data[offset2+j] = pImagePlane[offset1+j]*255;
						else
							im.data[offset2+j] = pImagePlane[offset1+j];
						break;
					case derivative:
						
						if(IsFloat)
							im.data[offset2+j] = (double)(pImagePlane[offset1+j]/Max+0.5)*255;
						else
							im.data[offset2+j] = ((double)pImagePlane[offset1+j]/Max+0.5)*255;
						break;
					case normalized:
						im.data[offset2+j] = (double)(pImagePlane[offset1+j]-Min)/(Max-Min)*255;
						break;
				}
			}
		}
	}*/
	return im;
}

template <class T>
bool ImageIO::saveImage(const char* filename,const T* pImagePlane,int width,int height, int nchannels,ImageType imtype)
{
	cv::Mat im = image2mat(pImagePlane, width, height,  nchannels, imtype);
	if(!im.empty())
		return cv::imwrite(filename,im);
	else
		return false;
}


#endif