// Copyright Joan Sol Roo and Christian Richardt (7 August 2014)
//
// solroo@gmail.com
// christian@richardt.name
//
// This software is a computer program whose purpose is to compute temporally
// coherent stereoscopic 3D videos from anaglyph 3D input videos.
//
// This software is governed by the CeCILL license under French law and
// abiding by the rules of distribution of free software. You can use, modify
// and/or redistribute the software under the terms of the CeCILL license as
// circulated by CEA, CNRS and INRIA at "http://www.cecill.info".
//
// As a counterpart to the access to the source code and rights to copy,
// modify and redistribute granted by the license, users are provided only
// with a limited warranty and the software's author, the holder of the
// economic rights, and the successive licensors have only limited liability.
//
// In this respect, the user's attention is drawn to the risks associated
// with loading, using, modifying and/or developing or reproducing the
// software by the user in light of its specific status of free software,
// that may mean that it is complicated to manipulate, and that also
// therefore means that it is reserved for developers and experienced
// professionals having in-depth computer knowledge. Users are therefore
// encouraged to load and test the software's suitability as regards their
// requirements in conditions enabling the security of their systems and/or
// data to be ensured and, more generally, to use and operate it in the same
// conditions as regards security.
//
// The fact that you are presently reading this means that you have had
// knowledge of the CeCILL license and that you accept its terms.

#include <FlowUtils.h>
#include <MatrixOperations.h>

#include "FarnebackFlow.h"
#include "SequenceFilter.h"

using namespace cv;
using namespace std;


Mat2f computeFlowFB(const Mat1f& image1, const Mat1f& image2)
{
	Mat flow;
	calcOpticalFlowFarneback(image1, image2, flow, 0.5, 3, 14, 3, 5, 1.2, 0);
	return flow;
}

vector<KeyFrame> calculateFlowFB(FrameSequence& frames)
{
	const size_t n = frames.size();
	vector<KeyFrame> time_flow(n);
	vector<Mat1b> gray = calculateGrayscale(frames);

	#pragma omp parallel for
	for(int i = 0; i < n; i++)
	{
		Mat3f& frame = frames[i];
		
		KeyFrame flow;
		flow.number = i;
		flow.current = &frame;

		if(i == 0)
		{
			flow.previous = NULL;
			flow.flow_backward = Mat2f::zeros(frame.rows, frame.cols);
		}
		else
		{
			// Relation with the previous frame
			flow.previous = &frames[i - 1];
			flow.flow_backward = computeFlowFB(gray[i], gray[i - 1]);
		}

		if(i + 1 < n)
		{
			// Relation with the next frame
			flow.next = &frames[i + 1];
			flow.flow_forward = computeFlowFB(gray[i], gray[i + 1]);
		}
		else
		{
			flow.next = NULL;
			flow.flow_forward = Mat2f::zeros(frame.rows, frame.cols);
		}
		
		time_flow[i] = flow;
	}

	return time_flow;
}
