// Copyright Joan Sol Roo and Christian Richardt (7 August 2014)
//
// solroo@gmail.com
// christian@richardt.name
//
// This software is a computer program whose purpose is to compute temporally
// coherent stereoscopic 3D videos from anaglyph 3D input videos.
//
// This software is governed by the CeCILL license under French law and
// abiding by the rules of distribution of free software. You can use, modify
// and/or redistribute the software under the terms of the CeCILL license as
// circulated by CEA, CNRS and INRIA at "http://www.cecill.info".
//
// As a counterpart to the access to the source code and rights to copy,
// modify and redistribute granted by the license, users are provided only
// with a limited warranty and the software's author, the holder of the
// economic rights, and the successive licensors have only limited liability.
//
// In this respect, the user's attention is drawn to the risks associated
// with loading, using, modifying and/or developing or reproducing the
// software by the user in light of its specific status of free software,
// that may mean that it is complicated to manipulate, and that also
// therefore means that it is reserved for developers and experienced
// professionals having in-depth computer knowledge. Users are therefore
// encouraged to load and test the software's suitability as regards their
// requirements in conditions enabling the security of their systems and/or
// data to be ensured and, more generally, to use and operate it in the same
// conditions as regards security.
//
// The fact that you are presently reading this means that you have had
// knowledge of the CeCILL license and that you accept its terms.

#pragma once

#include <vector>


struct TCParameters
{
	float sigma_s;// = 2000;
	float sigma_r;// = 0.4;
	int propagation_iterations;// = 3;
	float sigma_st;// = 5;
	float sigma_rt;// = 0.1;
	int iterations;// = 2;
	float theta;// = 5;
	bool filter;// = true
	float sigma_sv;// = 5;
	float sigma_rv;// = 0.1;
	float sigma_h;
	int filter_method;
	std::vector<float> luminance_scale;

	const static int kMethod_NC = 1;
	const static int kMethod_IC = 2;
	const static int kMethod_RF = 3;

	static TCParameters& off()
	{
		static TCParameters parameter;
		parameter.filter = false;
		parameter.iterations = 0;
		return parameter;
	}

	TCParameters(float _sigma_s = 2000, float _sigma_r = 0.4, int _propagation_iterations = 3, float _sigma_st = 5,  float _sigma_rt = 0.1, int _iterations = 2, float _theta = 5, float _sigma_sv = 20, float _sigma_rv = 0.6, int _filter_method = kMethod_NC)
	{
		sigma_s                = _sigma_s;
		sigma_r                = _sigma_r;
		propagation_iterations = _propagation_iterations;
		sigma_st               = _sigma_st;
		sigma_rt               = _sigma_rt;
		iterations             = _iterations;
		theta                  = _theta;
		sigma_sv               = _sigma_sv;
		sigma_rv               = _sigma_rv;

		filter = true;
		filter_method = _filter_method;
	}
};
