// Copyright Joan Sol Roo and Christian Richardt (7 August 2014)
//
// solroo@gmail.com
// christian@richardt.name
//
// This software is a computer program whose purpose is to compute temporally
// coherent stereoscopic 3D videos from anaglyph 3D input videos.
//
// This software is governed by the CeCILL license under French law and
// abiding by the rules of distribution of free software. You can use, modify
// and/or redistribute the software under the terms of the CeCILL license as
// circulated by CEA, CNRS and INRIA at "http://www.cecill.info".
//
// As a counterpart to the access to the source code and rights to copy,
// modify and redistribute granted by the license, users are provided only
// with a limited warranty and the software's author, the holder of the
// economic rights, and the successive licensors have only limited liability.
//
// In this respect, the user's attention is drawn to the risks associated
// with loading, using, modifying and/or developing or reproducing the
// software by the user in light of its specific status of free software,
// that may mean that it is complicated to manipulate, and that also
// therefore means that it is reserved for developers and experienced
// professionals having in-depth computer knowledge. Users are therefore
// encouraged to load and test the software's suitability as regards their
// requirements in conditions enabling the security of their systems and/or
// data to be ensured and, more generally, to use and operate it in the same
// conditions as regards security.
//
// The fact that you are presently reading this means that you have had
// knowledge of the CeCILL license and that you accept its terms.

#include <iomanip>
#include <ctime>

#include <MatrixOperations.h>

#include "VideoIO.h"

using namespace cv;
using namespace std;


VideoIO::VideoIO(string path, string extension, int num_digits)
	: path(path)
	, extension(extension)
	, num_digits(num_digits)
{
	resize = false;
	max_size = Size(600, 600);
}

FrameSequence VideoIO::loadSequence(string file, int start_at, int frames, int step)
{
	FrameSequence sequence(frames);

	#pragma omp parallel for
	for (int i = 0; i < frames; ++i)
	{
		int f = i * step + start_at;

		ostringstream ssP;
		ssP << std::setw(num_digits) << setfill('0') << f;
		string filename = path + file + ssP.str() + extension;

		Mat image_u8 = imread(filename);
		if(image_u8.empty())
		{
			printf("Error: Image \"%s\" could not be loaded.\n\n", filename.c_str());
			exit(-1);
		}

		Mat3f image = im2float(image_u8);
		if(resize && (image.cols > max_size.width || image.rows > max_size.height))
		{
			// Find the scale to resize while preserving the aspect ratio
			float scale = min(float(max_size.width) / image.cols, float(max_size.height) / image.rows);
			cv::resize(image, image, Size(int(image.cols * scale), int(image.rows * scale)), 0, 0, INTER_LINEAR);
		}
		sequence[i] = image;
	}

	return sequence;
}

void VideoIO::setResize(bool enabled)
{
	resize = enabled;
}

void VideoIO::setMaxSize(Size max_size)
{
	this->max_size = max_size;
	resize = (max_size != Size());
}
